@@ -4,7 +4,6 @@ from __future__ import annotations
 
 import asyncio
 import base64
-import gzip
 import hashlib
 import hmac
 import json
@@ -17,10 +16,8 @@ from random import randint
 from typing import Any, Callable, Coroutine, Optional
 
 import aiohttp
-from Crypto.Cipher import AES
-from Crypto.Util.Padding import unpad
 
-from .code_mappings import RoborockDockTypeCode, RoborockEnum
+from .code_mappings import RoborockDockTypeCode
 from .containers import (
     CleanRecord,
     CleanSummary,
@@ -42,10 +39,13 @@ from .exceptions import (
     RoborockException,
     RoborockInvalidCode,
     RoborockInvalidEmail,
+    RoborockInvalidUserAgreement,
+    RoborockNoUserAgreement,
     RoborockTimeout,
     RoborockUrlException,
     VacuumError,
 )
+from .protocol import Utils
 from .roborock_future import RoborockFuture
 from .roborock_message import RoborockMessage
 from .roborock_typing import DeviceProp, DockSummary, RoborockCommand
@@ -148,15 +148,13 @@ class RoborockClient:
                     payload = data.payload[0:24]
                     [endpoint, _, request_id, _] = struct.unpack("<15sBH6s", payload)
                     if endpoint.decode().startswith(self._endpoint):
-                        iv = bytes(AES.block_size)
-                        decipher = AES.new(self._nonce, AES.MODE_CBC, iv)
-                        decrypted = unpad(decipher.decrypt(data.payload[24:]), AES.block_size)
-                        decrypted = gzip.decompress(decrypted)
+                        decrypted = Utils.decrypt_cbc(data.payload[24:], self._nonce)
+                        decompressed = Utils.decompress(decrypted)
                         queue = self._waiting_queue.get(request_id)
                         if queue:
-                            if isinstance(decrypted, list):
-                                decrypted = decrypted[0]
-                            queue.resolve((decrypted, None))
+                            if isinstance(decompressed, list):
+                                decompressed = decompressed[0]
+                            queue.resolve((decompressed, None))
                 else:
                     queue = self._waiting_queue.get(data.seq)
                     if queue:
@@ -166,7 +164,7 @@ class RoborockClient:
 
     def on_connection_lost(self, exc: Optional[Exception]) -> None:
         self._last_disconnection = self.time_func()
-        _LOGGER.warning("Roborock client disconnected")
+        _LOGGER.info("Roborock client disconnected")
         if exc is not None:
             _LOGGER.warning(exc)
 
@@ -223,7 +221,10 @@ class RoborockClient:
     async def get_status(self) -> Status | None:
         status = await self.send_command(RoborockCommand.GET_STATUS)
         if isinstance(status, dict):
-            return Status.from_dict(status)
+            status = Status.from_dict(status)
+            status.update_status(self.device_info.model_specification)
+            return status
+
         return None
 
     async def get_dnd_timer(self) -> DNDTimer | None:
@@ -294,7 +295,7 @@ class RoborockClient:
             _LOGGER.error(e)
         return None
 
-    async def get_dock_summary(self, dock_type: RoborockEnum) -> DockSummary | None:
+    async def get_dock_summary(self, dock_type: RoborockDockTypeCode) -> DockSummary | None:
         """Gets the status summary from the dock with the methods available for a given dock.
 
         :param dock_type: RoborockDockTypeCode"""
@@ -306,7 +307,7 @@ class RoborockClient:
                     DustCollectionMode | WashTowelMode | SmartWashParams | None,
                 ]
             ] = [self.get_dust_collection_mode()]
-            if dock_type == RoborockDockTypeCode["3"]:
+            if dock_type == RoborockDockTypeCode.empty_wash_fill_dock or dock_type == RoborockDockTypeCode.s8_dock:
                 commands += [
                     self.get_wash_towel_mode(),
                     self.get_smart_wash_params(),
@@ -333,7 +334,7 @@ class RoborockClient:
         if clean_summary and clean_summary.records and len(clean_summary.records) > 0:
             last_clean_record = await self.get_clean_record(clean_summary.records[0])
         dock_summary = None
-        if status and status.dock_type is not None and status.dock_type != RoborockDockTypeCode["0"]:
+        if status and status.dock_type is not None and status.dock_type != RoborockDockTypeCode.no_dock:
             dock_summary = await self.get_dock_summary(status.dock_type)
         if any([status, dnd_timer, clean_summary, consumable]):
             return DeviceProp(
@@ -475,6 +476,12 @@ class RoborockApiClient:
         if response_code != 200:
             if response_code == 2018:
                 raise RoborockInvalidCode("Invalid code - check your code and try again.")
+            if response_code == 3009:
+                raise RoborockNoUserAgreement("You must accept the user agreement in the Roborock app to continue.")
+            if response_code == 3006:
+                raise RoborockInvalidUserAgreement(
+                    "User agreement must be accepted again - or you are attempting to use the Mi Home app account."
+                )
             raise RoborockException(f"{login_response.get('msg')} - response code: {response_code}")
         user_data = login_response.get("data")
         if not isinstance(user_data, dict):
